package com.sxtvs.open.api.odata.controller;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSON;
import com.sxtvs.open.api.odata.dto.RedirectDto;
import com.sxtvs.open.api.odata.dto.weibo.WeiboCallbackDto;
import com.sxtvs.open.api.odata.dto.wx.WxApiCreatePreAuthCodeResponse;
import com.sxtvs.open.api.odata.service.OauthService;
import com.sxtvs.open.api.odata.wxutil.WxCrypt;
import com.sxtvs.open.api.odata.wxutil.WxXmlUtil;
import com.sxtvs.open.core.advice.NoAPIResponse;
import com.sxtvs.open.core.auth.HttpContextUtil;
import com.sxtvs.open.core.auth.LoginRequired;
import com.sxtvs.open.core.conf.OauthConfig;
import com.sxtvs.open.core.sls.AliyunLogger;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping("oauth")
public class OauthRestController {

    @Autowired
    private AliyunLogger logger;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private OauthService oauthService;

    @RequestMapping("weixin/auth")
    @NoAPIResponse
    public String auth(@RequestBody String body) {
        logger.info("body", body, "key", "oauth/weixin/auth");
        var encrypt = WxXmlUtil.getEncrypt(body);
        logger.debug("data", encrypt);
        String xml = WxCrypt.decrypt(encrypt);
        logger.debug("data", xml);
        String componentVerifyTicket = WxXmlUtil.getComponentVerifyTicket(xml);
        logger.debug("data", componentVerifyTicket);
        redisTemplate.opsForValue().set("WX_TICKET", componentVerifyTicket);
        return "success";
    }

    @RequestMapping("douyin/code")
    @LoginRequired
    public RedirectDto douyinQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/douyin/code", "state", state);
        oauthService.saveStateMapping(token, state);
        return new RedirectDto("https://open.douyin.com/platform/oauth/connect/?" +
                "client_key=awfrca76s1petwh0&response_type=code&" +
                "scope=video.data%2Cfans.list%2Cdata.external.item%2Cdata.external.user%2Cfans.data%2Cvideo.list%2Crenew_refresh_token%2Cuser_info&" +
                "redirect_uri=https%3A%2F%2Fopen.sxtvs.net%2Fdouyin%2Fcallback-open&state=" + state);
    }

    @RequestMapping("kuaishou/code")
    @LoginRequired
    public RedirectDto kuaishouQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/douyin/code", "state", state, "token", token);
        oauthService.saveStateMapping(token, state);
        return new RedirectDto("http://open.kuaishou.com/oauth2/connect?" +
                "app_id=" + OauthConfig.kuaishouClientKey + "&response_type=code&" +
                "scope=user_info%2Cuser_video_info&" +
                "redirect_uri=http%3A%2F%2Fopen.sxtvs.net%2Fkuaishou%2Fcallback&state=" + state);

    }

    @RequestMapping("toutiao/code")
    @LoginRequired
    public RedirectDto toutiaoQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/toutiao/code", "state", state, "token", token);
        return new RedirectDto("https://open.snssdk.com/oauth/authorize/?" +
                "client_key=awfrca76s1petwh0&response_type=code&" +
                "scope=toutiao.video.data,user_info&" +
                "redirect_uri=https%3A%2F%2Fopen.sxtvs.net%2Ftoutiao%2Fcallback-open&state=" + state);
    }

    @RequestMapping("weibo/code")
    @LoginRequired
    public RedirectDto weiboQrCode() {
        var weiboUrl = oauthService.getWeiboUrl();
        logger.info("key", "weiboQrCode", "url", weiboUrl);
        return new RedirectDto("wait 还得找厂家对");
    }

    @RequestMapping("weibo/callback")
    public void weiboCallback(WeiboCallbackDto dto, HttpServletRequest request) {
        logger.info("key", "oauth/weibo/callback", "data", JSON.toJSONString(dto));
    }

    @RequestMapping("bilibili/code")
    @LoginRequired
    public RedirectDto bilibiliQrCode() {
        var state = IdUtil.nanoId();
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/bilibili/code", "state", state, "token", token);
        return new RedirectDto("https://passport.bilibili.com/register/pc_oauth2.html#/?client_id=e79e6fdef9d446d7" +
                "&return_url=https%3A%2F%2Fopen.sxtvs.net%2Fbilibili%2Fcallback-open&response_type=code&state=" + state);
    }

    @RequestMapping("weixin/code")
    @LoginRequired
    public RedirectDto weixinQrCode() {
        var token = HttpContextUtil.mustToken();
        logger.info("key", "oauth/weixin/code", "token", token);
        WxApiCreatePreAuthCodeResponse preAuthCodeResponse = oauthService.createPreAuthCodeResponse();
        // 这里使用预授权码跟用户token做一个映射
        var preAuthCode = preAuthCodeResponse.getPreAuthCode();
        oauthService.saveStateMapping(token, preAuthCode);

        return new RedirectDto("https://mp.weixin.qq.com/cgi-bin/componentloginpage?component_appid=wx7040933fd0e4b0e8&pre_auth_code="
                + preAuthCode + "&redirect_uri=http%3A%2F%2Fopen.sxtvs.net%2Foauth%2Fweixin%2Fwx7040933fd0e4b0e8%2Fcallback&auth_type=1");
    }


}
