package com.sxtvs.open.api.odata.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.http.HttpUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dao.BilibiliAccountMapper;
import com.sxtvs.open.api.odata.dto.bilibili.BilibiliAccessToken;
import com.sxtvs.open.api.odata.dto.bilibili.BilibiliAccountInfo;
import com.sxtvs.open.api.odata.dto.bilibili.BilibiliRefreshToken;
import com.sxtvs.open.api.odata.entity.BilibiliAccount;
import com.sxtvs.open.core.conf.Constant;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;

@Service
@Slf4j
public class BilibiliAccountService extends ServiceImpl<BilibiliAccountMapper, BilibiliAccount> {



    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private QrcodeLogServiceImpl qrcodeLogService;


    @SneakyThrows
    private void renewAccessToken(BilibiliAccount bilibiliAccount) {
        HashMap<String, Object> param = new HashMap<String, Object>() {{
            this.put("client_id", Constant.bilibiliClientId);
            this.put("client_secret", Constant.bilibiliSecret);
            this.put("grant_type", "refresh_token");
            this.put("refresh_token", bilibiliAccount.getRefreshToken());
        }};
        String body = HttpUtil.post("https://api.bilibili.com/x/account-oauth2/v1/refresh_token", param);
        BilibiliRefreshToken bilibiliRefreshToken = objectMapper.readValue(body, BilibiliRefreshToken.class);
        BilibiliRefreshToken.DataDTO data = bilibiliRefreshToken.getData();
        bilibiliAccount.setRefreshToken(data.getRefreshToken());
        bilibiliAccount.setAccessToken(data.getAccessToken());
        bilibiliAccount.setAtExpireTime(new Date(data.getExpiresIn() * 1000));
        bilibiliAccount.setRtExpireTime(new Date(data.getExpiresIn() * 1000));
        bilibiliAccount.setUpdateTime(new Date());
        this.updateById(bilibiliAccount);
        log.info("renewAccessToken {}", bilibiliAccount);

    }

    @SneakyThrows
    public BilibiliAccount upsert(String code) {
        HashMap<String, Object> param = new HashMap<String, Object>() {{
            this.put("client_id", Constant.bilibiliClientId);
            this.put("client_secret", Constant.bilibiliSecret);
            this.put("grant_type", "authorization_code");
            this.put("code", code);
        }};
        String body = HttpUtil.post("https://api.bilibili.com/x/account-oauth2/v1/token", param);
        log.info("bilibili token {}",body);
        BilibiliAccessToken bilibiliAccessToken = objectMapper.readValue(body, BilibiliAccessToken.class);

        BilibiliAccessToken.DataDTO data = bilibiliAccessToken.getData();
        String accessToken = data.getAccessToken();
        String refreshToken = data.getRefreshToken();
        Long expiresIn = data.getExpiresIn();


        param.clear();
        param.put("client_id",Constant.bilibiliClientId);
        param.put("access_token",accessToken);
        body = HttpUtil.get("http://member.bilibili.com/arcopen/fn/user/account/info",param);
        log.info("bilibili info {}",body);
        BilibiliAccountInfo bilibiliAccountInfo = objectMapper.readValue(body, BilibiliAccountInfo.class);
        BilibiliAccountInfo.DataDTO infoData = bilibiliAccountInfo.getData();
        String openId = infoData.getOpenid();
        String name = infoData.getName().trim();

        qrcodeLogService.writeLog("B站",openId,name);

        BilibiliAccount bilibiliAccount = this.getById(openId);
        DateTime now = new DateTime();
        if (bilibiliAccount == null) {
            bilibiliAccount =  BilibiliAccount.builder()
                        .openId(openId)
                        .atExpireTime(new Date(expiresIn*1000))
                        .rtExpireTime(new Date(expiresIn*1000))
                        .accessToken(accessToken)
                        .refreshToken(refreshToken)
                        .nickName(name)
                        .updateTime(now.toJdkDate())
                        .createTime(now.toJdkDate())
                        .build();
            this.save(bilibiliAccount);
            log.info("bilibili {} 新增成功", bilibiliAccount);
        } else {
                bilibiliAccount.setAtExpireTime(new Date(expiresIn*1000));
                bilibiliAccount.setRtExpireTime(new Date(expiresIn*1000));
                bilibiliAccount.setAccessToken(accessToken);
                bilibiliAccount.setRefreshToken(refreshToken);
                bilibiliAccount.setNickName(name);
                bilibiliAccount.setUpdateTime(now.toJdkDate());
                this.updateById(bilibiliAccount);

            log.info("bilibili {} 更新成功", bilibiliAccount);
        }

//
        return bilibiliAccount;

    }


    public void refresh() {
        Date expireTime = new DateTime()
                .offset(DateField.DAY_OF_YEAR, 2)
                .toJdkDate();

        //检测快过期的token
        for (BilibiliAccount bilibiliAccount : this.lambdaQuery()
                .le(BilibiliAccount::getAtExpireTime, expireTime)
                .list()) {
            renewAccessToken(bilibiliAccount);
        }


    }


    public static void main(String[] args) {
        Date dateTime = new Date(1655104166L * 1000);
        System.out.println(dateTime);
    }


}
