package com.sxtvs.open.api.odata.service;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.http.HttpUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.sxtvs.open.api.odata.dto.wx.*;
import com.sxtvs.open.api.odata.dto.wx.apiquery.AuthorizationInfo;
import com.sxtvs.open.api.odata.dto.wx.apiquery.WxApiQueryAuthRequest;
import com.sxtvs.open.api.odata.dto.wx.apiquery.WxApiQueryAuthResponse;
import com.sxtvs.open.api.odata.dto.wx.authorization.AuthorizationInfoResponse;
import com.sxtvs.open.api.odata.entity.WxAccount;
import com.sxtvs.open.api.odata.entity.WxAccountV2;
import com.sxtvs.open.core.conf.Constant;
import com.sxtvs.open.core.http.RequestUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Date;

import static com.sxtvs.open.core.conf.Constant.weixinZhiHuiRongMeiAppId;
import static com.sxtvs.open.core.conf.Constant.weixinZhiHuiRongMeiSecret;

@Service
@Slf4j
public class WxRestService {

    @Autowired
    @Qualifier("weixinRestTemplate")
    private RestTemplate template;

    @Autowired
    private RequestUtil requestUtil;

    @Autowired
    private com.sxtvs.open.api.odata.service.WxTicketService wxTicketService;

    @Autowired
    private WxAccountService wxAccountService;

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private WxAccountV2Service wxAccountV2Service;

    @Autowired
    private QrcodeLogServiceImpl qrcodeLogService;

    public WxApiComponentTokenResponse refreshComponentToken() {
        WxApiComponentTokenRequest tokenRequest = new WxApiComponentTokenRequest();
        tokenRequest.setComponentAppid(Constant.wxClientKey);
        tokenRequest.setComponentAppsecret(Constant.wxClientSecret);
        tokenRequest.setComponentVerifyTicket(wxTicketService.getTicket());
        WxApiComponentTokenResponse tokenResponse = template.postForObject(
                "https://api.weixin.qq.com/cgi-bin/component/api_component_token",
                requestUtil.buildJsonObject(tokenRequest), WxApiComponentTokenResponse.class);
        log.info("refreshComponentToken {}", tokenResponse);
        wxTicketService.updateComponentAccessToken(tokenResponse.getComponentAccessToken(),
                tokenResponse.getExpiresIn());
        return tokenResponse;
    }


    public WxApiCreatePreAuthCodeResponse createPreAuthCodeResponse() {
        String componentAccessToken = wxTicketService.getTicketObject().getComponentAccessToken();
        WxApiCreatePreAuthCodeRequest authCodeRequest = new WxApiCreatePreAuthCodeRequest();
        authCodeRequest.setComponentAppid(Constant.wxClientKey);
        authCodeRequest.setComponentAccessToken(componentAccessToken);
        WxApiCreatePreAuthCodeResponse preAuthCodeResponse = template.postForObject(
                "https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?" +
                        "component_access_token=" + componentAccessToken,
                requestUtil.buildJsonObject(authCodeRequest), WxApiCreatePreAuthCodeResponse.class);
        log.info("{}", preAuthCodeResponse);
        return preAuthCodeResponse;
    }

    public void apiQueryAuth(String authCode) {
        String componentAccessToken = wxTicketService.getTicketObject().getComponentAccessToken();
        WxApiQueryAuthRequest authCodeRequest = new WxApiQueryAuthRequest();
        authCodeRequest.setComponentAppid(Constant.wxClientKey);
        authCodeRequest.setComponentAccessToken(componentAccessToken);
        authCodeRequest.setAuthorizationCode(authCode);
        WxApiQueryAuthResponse wxApiQueryAuthResponse = template.postForObject(
                "https://api.weixin.qq.com/cgi-bin/component/api_query_auth?" +
                        "component_access_token=" + componentAccessToken,
                requestUtil.buildJsonObject(authCodeRequest), WxApiQueryAuthResponse.class);
        log.info("{}", wxApiQueryAuthResponse);
        upsertWxAccount(wxApiQueryAuthResponse);
    }

    public AuthorizationInfoResponse getAuthorizerInfo(String openId) {
        WxApiGetAuthorizerInfoRequest infoRequest = new WxApiGetAuthorizerInfoRequest();
        infoRequest.setComponentAppid(Constant.wxClientKey);
        String componentAccessToken = wxTicketService.getTicketObject().getComponentAccessToken();
        infoRequest.setComponentAccessToken(componentAccessToken);
        infoRequest.setAuthorizerAppid(openId);

        AuthorizationInfoResponse authorizationInfoResponse = template.postForObject(
                "https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_info?" +
                        "component_access_token=" + componentAccessToken,
                requestUtil.buildJsonObject(infoRequest), AuthorizationInfoResponse.class);

        return authorizationInfoResponse;

    }

    public synchronized void upsertWxAccount(WxApiQueryAuthResponse wxApiQueryAuthResponse) {

        AuthorizationInfo authorizationInfo = wxApiQueryAuthResponse.getAuthorizationInfo();
        String openId = authorizationInfo.getAuthorizerAppid();
        String ak = authorizationInfo.getAuthorizerAccessToken();
        String rk = authorizationInfo.getAuthorizerRefreshToken();
        int expiresIn = authorizationInfo.getExpiresIn();
        String nickName = getAuthorizerInfo(openId).getAuthorizerInfo().getNickName();

        qrcodeLogService.writeLog("微信", openId, nickName);
        WxAccount wxAccount = wxAccountService.getById(openId);

        DateTime now = new DateTime();
        now.setMutable(false);
        if (wxAccount == null) {
            wxAccount = new WxAccount();
            wxAccount.setNickName(nickName);
            wxAccount.setOpenId(openId);
            wxAccount.setAccessToken(ak);
            wxAccount.setAtExpireTime(now.offset(DateField.SECOND, expiresIn));
            wxAccount.setRefreshToken(rk);
            wxAccount.setRtExpireTime(now.offset(DateField.YEAR, 3));
            wxAccount.setCreateTime(now);
            wxAccount.setUpdateTime(now);
            wxAccountService.save(wxAccount);
        } else {
            wxAccount.setAccessToken(ak);
            wxAccount.setNickName(nickName);
            wxAccount.setAtExpireTime(now.offset(DateField.SECOND, expiresIn));
            wxAccount.setRefreshToken(rk);
            wxAccount.setRtExpireTime(now.offset(DateField.YEAR, 3));
            wxAccount.setUpdateTime(now);
            wxAccountService.updateById(wxAccount);
        }
        log.info("wx nick_name {}", wxAccount.getNickName());
    }

    @SneakyThrows
    public void refreshAllAccessToken() {
        DateTime now = new DateTime();
        now.setMutable(false);
        Date expireTime = new DateTime()
                .offset(DateField.MINUTE, 20)
                .toJdkDate();
        String componentAccessToken = wxTicketService.getTicketObject().getComponentAccessToken();
        //检测快过期的at
        for (WxAccount wxAccount : wxAccountService.lambdaQuery()
                .le(WxAccount::getAtExpireTime, expireTime)
                .list()) {

            String openId = wxAccount.getOpenId();

            WxApiAuthorizerTokenRequest tokenRequest = new WxApiAuthorizerTokenRequest();
            tokenRequest.setComponentAppid(Constant.wxClientKey);
            tokenRequest.setComponentAccessToken(componentAccessToken);
            tokenRequest.setAuthorizerRefreshToken(wxAccount.getRefreshToken());
            tokenRequest.setAuthorizerAppid(openId);
            String response = HttpUtil.post("https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?" +
                            "component_access_token=" + componentAccessToken,
                    objectMapper.writeValueAsString(tokenRequest));
            log.info("refreshAllAccessToken openId {} tokenResponse {}", openId,response);
            WxApiAuthorizerTokenResponse tokenResponse;
            try {
                tokenResponse = objectMapper.readValue(response, WxApiAuthorizerTokenResponse.class);
                if (tokenResponse.hasError()) {
                    log.error("refreshAllAccessToken openId {} tokenResponse {}", openId,response);
                    continue;
                }
                wxAccount.setAccessToken(tokenResponse.getAuthorizerAccessToken());
                wxAccount.setRefreshToken(tokenResponse.getAuthorizerRefreshToken());
                wxAccount.setUpdateTime(now);
                wxAccount.setRtExpireTime(now.offset(DateField.YEAR, 3));
                wxAccount.setAtExpireTime(now.offset(DateField.SECOND, tokenResponse.getExpiresIn()));
                wxAccountService.updateById(wxAccount);
            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
//            WxApiAuthorizerTokenResponse tokenResponse = template.postForObject(
//                    "https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?" +
//                            "component_access_token=" + componentAccessToken,
//                    requestUtil.buildJsonObject(tokenRequest), WxApiAuthorizerTokenResponse.class);

        }

        //刷新v2表的数据
        for (WxAccountV2 wxAccountV2 : wxAccountV2Service.lambdaQuery()
                .le(WxAccountV2::getAtExpireTime, expireTime)
                .list()) {
            String openId = wxAccountV2.getOpenId();
            String secret = wxAccountV2.getSecret();
            String response = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + openId + "&secret=" + secret);
            log.info("refreshAllAccessTokenV2 appid {} response {}", openId,response);
            try {
                JsonNode jsonNode = objectMapper.readTree(response);
                String accessToken = jsonNode.get("access_token").asText();
                int expiresIn = jsonNode.get("expires_in").asInt();
                wxAccountV2.setRefreshToken("");
                wxAccountV2.setAccessToken(accessToken);
                wxAccountV2.setUpdateTime(new Date());
                wxAccountV2.setRtExpireTime(now.offset(DateField.YEAR, 3));
                wxAccountV2.setAtExpireTime(now.offset(DateField.SECOND, expiresIn));
                wxAccountV2Service.updateById(wxAccountV2);
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

    }

    @SneakyThrows
    public void refreshJsTicket() {
        Date expireTime = new DateTime()
                .offset(DateField.MINUTE, 20)
                .toJdkDate();
        for (WxAccountV2 wxAccountV2 : wxAccountV2Service.lambdaQuery()
                .le(WxAccountV2::getJsTicketExpireTime, expireTime)
                .list()) {
            String accessToken = wxAccountV2.getAccessToken();
            String response = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=" + accessToken + "&type=jsapi");
            log.info("refreshJsTicket response {}",response);
            JsonNode node = objectMapper.readTree(response).get("ticket");
            if (node == null) {
                log.error("refreshJsTicket response {}",response);
                continue;
            }
            String ticket = node.asText();
            wxAccountV2.setJsTicket(ticket);
            wxAccountV2.setJsTicketExpireTime(new DateTime()
                    .offset(DateField.SECOND, 7200)
                    .toJdkDate());
            wxAccountV2.setUpdateTime(new Date());
            wxAccountV2Service.updateById(wxAccountV2);
        }

    }


    @SneakyThrows
    private static void getJsTicket(String accessToken) {
        String response = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=" + accessToken + "&type=jsapi");
        ObjectMapper objectMapper = new ObjectMapper();
        String ticket = objectMapper.readTree(response).get("ticket").asText();
        System.out.println(ticket);
    }

    public static void main(String[] args) throws JsonProcessingException {
        String response = HttpUtil.get("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + weixinZhiHuiRongMeiAppId + "&secret=" + weixinZhiHuiRongMeiSecret);
        ObjectMapper objectMapper = new ObjectMapper();
        String access_token = objectMapper.readTree(response).get("access_token").asText();
        System.out.println(access_token);
//        String ak = "49_tJph-qS1EkuDtWcnSQ6JdNGttq4ywCo0jtcY52Z01oFwuPn0T6qsNL8Hqab8fPF4-GY-dpy0WblVv3Aos3NlAxrpQOs8qrYhEm5WvZE0Pb9q7mF031oScoUgvV9WmmNwoHOrgOIfAKeP0lh3RVVhABAYCC";
        getJsTicket(access_token);
    }


}
