package com.sxtvs.open.core.http;

import com.sxtvs.open.core.advice.APIResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.List;

@RestControllerAdvice
@Slf4j
public class APIResponseAdvice  {

    @ExceptionHandler(MethodArgumentNotValidException.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public APIResponse validError(MethodArgumentNotValidException ex) {
        log.error("校验未通过", ex);
        BindingResult result = ex.getBindingResult();
        StringBuilder builder = new StringBuilder();
        if (result.hasErrors()) {

            List<ObjectError> errors = result.getAllErrors();

            errors.forEach(p -> {

                FieldError fieldError = (FieldError) p;
                builder.append(fieldError.getDefaultMessage()).append(";");
                log.error("Data check failure : object{" + fieldError.getObjectName() + "},field{" + fieldError.getField() +
                        "},errorMessage{" + fieldError.getDefaultMessage() + "}");

            });

        }

        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);

        apiResponse.setMessage(builder.toString());
        return apiResponse;
    }

    /**
     * 拦截未知的运行时异常
     *
     * @author fengshuonan
     * @date 2020/12/16 15:12
     */
    @ExceptionHandler(Throwable.class)
    @ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
    public APIResponse serverError(Throwable ex) {
        log.error("服务器运行异常", ex);
        APIResponse apiResponse = new APIResponse();
        apiResponse.setCode(-1);
        apiResponse.setMessage(ex.getMessage());
        return apiResponse;
    }

}
